@@ -32,6 +32,8 @@
 #include <vector>
 #include <algorithm>
 
+#include "common_utils.h"
+
 namespace heif_examples {
   void show_version()
   {
@@ -121,7 +123,7 @@ namespace heif_examples {
 
     std::array<uint8_t, 4> length{};
     istr.read((char*) length.data(), length.size());
-    uint32_t box_size = (length[0] << 24) + (length[1] << 16) + (length[2] << 8) + (length[3]);
+    uint32_t box_size = four_bytes_to_uint32(length[0], length[1], length[2], length[3]);
     if ((box_size < 16) || (box_size > 512)) {
       fprintf(stderr, "Input file does not appear to start with a valid box length.");
       if ((box_size & 0xFFFFFFF0) == 0xFFD8FFE0) {
