@@ -47,11 +47,11 @@ public:
 
   bool supports_alpha() const override { return false; }
 
-  void UpdateDecodingOptions(const struct heif_image_handle* handle,
-                             struct heif_decoding_options* options) const override;
+  void UpdateDecodingOptions(const heif_image_handle* handle,
+                             heif_decoding_options* options) const override;
 
-  bool Encode(const struct heif_image_handle* handle,
-              const struct heif_image* image, const std::string& filename) override;
+  bool Encode(const heif_image_handle* handle,
+              const heif_image* image, const std::string& filename) override;
 
 private:
 };
