@@ -25,16 +25,42 @@
 #include "context.h"
 
 #include <memory>
+#include <vector>
+#include <string>
+#include "image-items/image_item.h"
 
 struct heif_image_handle
 {
-  std::shared_ptr<HeifContext::Image> image;
+  std::shared_ptr<ImageItem> image;
 
   // store reference to keep the context alive while we are using the handle (issue #147)
   std::shared_ptr<HeifContext> context;
 };
 
 
+struct heif_track
+{
+  std::shared_ptr<Track> track;
+
+  // store reference to keep the context alive while we are using the handle (issue #147)
+  std::shared_ptr<HeifContext> context;
+};
+
+struct heif_raw_sequence_sample
+{
+  ~heif_raw_sequence_sample()
+  {
+    heif_tai_timestamp_packet_release(timestamp);
+  }
+
+  std::vector<uint8_t> data;
+  uint32_t duration = 0;
+
+  heif_tai_timestamp_packet* timestamp = nullptr;
+  std::string gimi_sample_content_id;
+};
+
+
 struct heif_image
 {
   std::shared_ptr<HeifPixelImage> image;
@@ -49,11 +75,11 @@ struct heif_context
 
 struct heif_encoder
 {
-  heif_encoder(const struct heif_encoder_plugin* plugin);
+  explicit heif_encoder(const heif_encoder_plugin* plugin);
 
   ~heif_encoder();
 
-  struct heif_error alloc();
+  heif_error alloc();
 
   void release();
 
@@ -78,4 +104,11 @@ struct heif_region
   std::shared_ptr<RegionGeometry> region;
 };
 
+struct heif_text_item
+{
+  std::shared_ptr<HeifContext> context;
+  std::shared_ptr<TextItem> text_item;
+};
+
+
 #endif
