@@ -24,7 +24,9 @@
 #include <cstring>
 #include <cassert>
 
-#if ((defined(__GNUC__) && !defined(__clang__) && !defined(__INTEL_COMPILER) && !defined(__PGI)) && __GNUC__ < 9) || (defined(__clang__) && __clang_major__ < 10)
+#include "common_utils.h"
+
+#if !defined(HAVE_BIT)
 #include <type_traits>
 #else
 #include <bit>
@@ -268,10 +270,7 @@ uint32_t BitstreamRange::read32()
     return 0;
   }
 
-  return (uint32_t) ((buf[0] << 24) |
-                     (buf[1] << 16) |
-                     (buf[2] << 8) |
-                     (buf[3]));
+  return four_bytes_to_uint32(buf[0], buf[1], buf[2], buf[3]);
 }
 
 
@@ -360,7 +359,7 @@ float BitstreamRange::read_float32()
   // TODO: I am not sure this works everywhere as there seem to be systems where
   //       the float byte order is different from the integer endianness
   //       https://en.wikipedia.org/wiki/Endianness#Floating_point
-  int i = read32();
+  uint32_t i = read32();
   float f;
   memcpy(&f, &i, sizeof(float));
   return f;
@@ -395,12 +394,13 @@ std::string BitstreamRange::read_string()
     return std::string();
   }
 
+  auto istr = get_istream();
+
   for (;;) {
     if (!prepare_read(1)) {
       return std::string();
     }
 
-    auto istr = get_istream();
     char c;
     bool success = istr->read(&c, 1);
 
@@ -409,7 +409,7 @@ std::string BitstreamRange::read_string()
       return std::string();
     }
 
-    if (c == 0) {
+    if (c == 0 || m_remaining==0) {
       break;
     }
     else {
@@ -421,6 +421,55 @@ std::string BitstreamRange::read_string()
 }
 
 
+std::string BitstreamRange::read_fixed_string(int len)
+{
+  std::string str;
+
+  if (!prepare_read(len)) {
+    return std::string();
+  }
+
+  auto istr = get_istream();
+
+  uint8_t n;
+  bool success = istr->read(&n, 1);
+  if (!success || n > len - 1) {
+    return {};
+  }
+
+  for (int i = 0; i < n; i++) {
+    char c;
+    success = istr->read(&c, 1);
+
+    if (!success) {
+      set_eof_while_reading();
+      return std::string();
+    }
+
+    str += (char) c;
+  }
+
+  istr->seek_cur(len-n-1);
+
+  return str;
+}
+
+
+std::string BitstreamRange::read_string_until_eof()
+{
+  size_t n = get_remaining_bytes();
+
+  [[maybe_unused]] bool success = prepare_read(n);
+  assert(success); // we are reading exactly the rest of the box
+
+  std::string str;
+  str.resize(n);
+  get_istream()->read(str.data(), n);
+
+  return str;
+}
+
+
 bool BitstreamRange::read(uint8_t* data, size_t n)
 {
   if (!prepare_read(n)) {
@@ -487,9 +536,10 @@ void BitstreamRange::skip_without_advancing_file_pos(size_t n)
 
 
 BitReader::BitReader(const uint8_t* buffer, int len)
+  : data_start(buffer),
+    data_length(len)
 {
   data = buffer;
-  data_length = len;
   bytes_remaining = len;
 
   nextbits = 0;
@@ -499,6 +549,18 @@ BitReader::BitReader(const uint8_t* buffer, int len)
 }
 
 
+void BitReader::reset()
+{
+  data = data_start;
+  bytes_remaining = data_length;
+
+  nextbits = 0;
+  nextbits_cnt = 0;
+
+  refill();
+}
+
+
 uint32_t BitReader::get_bits(int n)
 {
   assert(n <= 32);
@@ -830,9 +892,9 @@ void StreamWriter::write(int size, uint64_t value)
 }
 
 
-void StreamWriter::write(const std::string& str)
+void StreamWriter::write(const std::string& str, bool end_with_null)
 {
-  size_t required_size = m_position + str.size() + 1;
+  size_t required_size = m_position + str.size() + (end_with_null ? 1 : 0);
 
   if (required_size > m_data.size()) {
     m_data.resize(required_size);
@@ -842,7 +904,31 @@ void StreamWriter::write(const std::string& str)
     m_data[m_position++] = str[i];
   }
 
-  m_data[m_position++] = 0;
+  if (end_with_null) {
+    m_data[m_position++] = 0;
+  }
+}
+
+
+void StreamWriter::write_fixed_string(std::string s, size_t len)
+{
+  size_t required_size = m_position + len;
+
+  if (required_size > m_data.size()) {
+    m_data.resize(required_size);
+  }
+
+  size_t n_chars = std::min(s.length(), len - 1);
+  assert(n_chars <= 255);
+  m_data[m_position++] = static_cast<uint8_t>(n_chars);
+
+  for (size_t i = 0; i < s.size() && i < len - 1; i++) {
+    m_data[m_position++] = s[i];
+  }
+
+  for (size_t i = s.size(); i < len - 1; i++) {
+    m_data[m_position++] = 0;
+  }
 }
 
 
