@@ -25,6 +25,7 @@
 
 #include <string>
 #include <algorithm>
+#include <utility>
 
 
 Result<std::vector<uint8_t>> Decoder_uncompressed::read_bitstream_configuration_data() const
@@ -175,11 +176,37 @@ Decoder_uncompressed::decode_single_frame_from_compressed_data(const struct heif
   auto decodeResult = UncompressedImageCodec::decode_uncompressed_image(properties,
                                                           get_data_extent(),
                                                           limits);
+  return decodeResult;
+}
 
-  if (decodeResult.error) {
-    return decodeResult.error;
+
+Error
+Decoder_uncompressed::decode_sequence_frame_from_compressed_data(bool upload_configuration_NALs,
+                                                                 const heif_decoding_options& options,
+                                                                 uintptr_t user_data,
+                                                                 const heif_security_limits* limits)
+{
+  auto decodingResult = decode_single_frame_from_compressed_data(options, limits);
+  if (decodingResult.error()) {
+    return decodingResult.error();
   }
-  else {
-    return decodeResult.value;
+
+  m_decoded_image = *decodingResult;
+  m_decoded_image_user_data = user_data;
+
+  return {};
+}
+
+Result<std::shared_ptr<HeifPixelImage> > Decoder_uncompressed::get_decoded_frame(const heif_decoding_options& options,
+                                                                                 uintptr_t* out_user_data,
+                                                                                 const heif_security_limits* limits)
+{
+  auto img = std::move(m_decoded_image);
+  m_decoded_image.reset();
+
+  if (out_user_data) {
+    *out_user_data = m_decoded_image_user_data;
   }
+
+  return img;
 }
