@@ -29,7 +29,8 @@ template<class Pixel>
 std::vector<ColorStateWithCost>
 Op_YCbCr_to_RGB<Pixel>::state_after_conversion(const ColorState& input_state,
                                                const ColorState& target_state,
-                                               const heif_color_conversion_options& options) const
+                                               const heif_color_conversion_options& options,
+                                               const heif_color_conversion_options_ext& options_ext) const
 {
   // this Op only implements the nearest-neighbor algorithm
 
@@ -47,8 +48,8 @@ Op_YCbCr_to_RGB<Pixel>::state_after_conversion(const ColorState& input_state,
     return {};
   }
 
-  int matrix = input_state.nclx_profile.get_matrix_coefficients();
-  if ( matrix == 11 || matrix == 14) {
+  int matrix = input_state.nclx.get_matrix_coefficients();
+  if (matrix == 11 || matrix == 14) {
     return {};
   }
 
@@ -64,6 +65,10 @@ Op_YCbCr_to_RGB<Pixel>::state_after_conversion(const ColorState& input_state,
     return {};
   }
 
+  if (input_state.bits_per_pixel > 14) {
+    return {};
+  }
+
   std::vector<ColorStateWithCost> states;
 
   ColorState output_state;
@@ -87,6 +92,7 @@ Op_YCbCr_to_RGB<Pixel>::convert_colorspace(const std::shared_ptr<const HeifPixel
                                            const ColorState& input_state,
                                            const ColorState& target_state,
                                            const heif_color_conversion_options& options,
+                                           const heif_color_conversion_options_ext& options_ext,
                                            const heif_security_limits* limits) const
 {
   bool hdr = !std::is_same<Pixel, uint8_t>::value;
@@ -127,8 +133,6 @@ Op_YCbCr_to_RGB<Pixel>::convert_colorspace(const std::shared_ptr<const HeifPixel
   }
 
 
-  auto colorProfile = input->get_color_profile_nclx();
-
   uint32_t width = input->get_width();
   uint32_t height = input->get_height();
 
@@ -176,7 +180,9 @@ Op_YCbCr_to_RGB<Pixel>::convert_colorspace(const std::shared_ptr<const HeifPixel
   }
 
 
-  uint16_t halfRange = (uint16_t) (1 << (bpp_y - 1));
+  uint16_t halfRange = static_cast<uint16_t>(1 << (bpp_y - 1));
+  int16_t halfRange_chroma = static_cast<int16_t>(1 << (bpp_cb - 1));
+
   int32_t fullRange = (1 << bpp_y) - 1;
   int limited_range_offset_int = 16 << (bpp_y - 8);
   float limited_range_offset = static_cast<float>(limited_range_offset_int);
@@ -196,11 +202,13 @@ Op_YCbCr_to_RGB<Pixel>::convert_colorspace(const std::shared_ptr<const HeifPixel
   int matrix_coeffs = 2;
   bool full_range_flag = true;
   YCbCr_to_RGB_coefficients coeffs = YCbCr_to_RGB_coefficients::defaults();
-  if (colorProfile) {
-    matrix_coeffs = colorProfile->get_matrix_coefficients();
-    full_range_flag = colorProfile->get_full_range_flag();
-    coeffs = get_YCbCr_to_RGB_coefficients(colorProfile->get_matrix_coefficients(),
-                                           colorProfile->get_colour_primaries());
+  if (input->has_nclx_color_profile()) {
+    nclx_profile colorProfile = input->get_color_profile_nclx();
+
+    matrix_coeffs = colorProfile.get_matrix_coefficients();
+    full_range_flag = colorProfile.get_full_range_flag();
+    coeffs = get_YCbCr_to_RGB_coefficients(colorProfile.get_matrix_coefficients(),
+                                           colorProfile.get_colour_primaries());
   }
 
 
@@ -235,6 +243,24 @@ Op_YCbCr_to_RGB<Pixel>::convert_colorspace(const std::shared_ptr<const HeifPixel
         out_g[y * out_g_stride + x] = (Pixel) (clip_int_u8(yv + cb));
         out_b[y * out_b_stride + x] = (Pixel) (clip_int_u8(yv - cb - cr));
       }
+      else if (matrix_coeffs == 16) {
+        int16_t yy = in_y[y * in_y_stride + x];
+        int16_t cb = static_cast<int16_t>(in_cb[cy * in_cb_stride + cx]) - halfRange_chroma;
+        int16_t cr = static_cast<int16_t>(in_cr[cy * in_cr_stride + cx]) - halfRange_chroma;
+
+        int16_t t = yy - (cb >> 1);
+        int16_t g = t + cb;
+        int16_t b = t - (cr>>1);
+        int16_t r = b + cr;
+
+        // TODO: we are extending the output RGB bpp by 2 bits because this function cannot do bit-depth
+        //       conversion yet. This should be ultimately replaced by a new function with implicit bit-depth reduction.
+
+        uint16_t max_rgb = static_cast<uint16_t>((1<<bpp_y)-1);
+        out_r[y * out_r_stride + x] = static_cast<Pixel>(clip_int_u16(r * 4, max_rgb));
+        out_g[y * out_g_stride + x] = static_cast<Pixel>(clip_int_u16(g * 4, max_rgb));
+        out_b[y * out_b_stride + x] = static_cast<Pixel>(clip_int_u16(b * 4, max_rgb));
+      }
       else { // TODO: matrix_coefficients = 11,14
         float yv, cb, cr;
         yv = static_cast<float>(in_y[y * in_y_stride + x] );
@@ -269,7 +295,8 @@ template class Op_YCbCr_to_RGB<uint16_t>;
 std::vector<ColorStateWithCost>
 Op_YCbCr420_to_RGB24::state_after_conversion(const ColorState& input_state,
                                              const ColorState& target_state,
-                                             const heif_color_conversion_options& options) const
+                                             const heif_color_conversion_options& options,
+                                             const heif_color_conversion_options_ext& options_ext) const
 {
   // this Op only implements the nearest-neighbor algorithm
 
@@ -287,11 +314,11 @@ Op_YCbCr420_to_RGB24::state_after_conversion(const ColorState& input_state,
     return {};
   }
 
-  int matrix = input_state.nclx_profile.get_matrix_coefficients();
+  int matrix = input_state.nclx.get_matrix_coefficients();
   if (matrix == 0 || matrix == 8 || matrix == 11 || matrix == 14) {
     return {};
   }
-  if (!input_state.nclx_profile.get_full_range_flag()) {
+  if (!input_state.nclx.get_full_range_flag()) {
     return {};
   }
 
@@ -317,6 +344,7 @@ Op_YCbCr420_to_RGB24::convert_colorspace(const std::shared_ptr<const HeifPixelIm
                                          const ColorState& input_state,
                                          const ColorState& target_state,
                                          const heif_color_conversion_options& options,
+                                         const heif_color_conversion_options_ext& options_ext,
                                          const heif_security_limits* limits) const
 {
   if (input->get_bits_per_pixel(heif_channel_Y) != 8 ||
@@ -336,11 +364,11 @@ Op_YCbCr420_to_RGB24::convert_colorspace(const std::shared_ptr<const HeifPixelIm
     return err;
   }
 
-  auto colorProfile = input->get_color_profile_nclx();
   YCbCr_to_RGB_coefficients coeffs = YCbCr_to_RGB_coefficients::defaults();
-  if (colorProfile) {
-    coeffs = get_YCbCr_to_RGB_coefficients(colorProfile->get_matrix_coefficients(),
-                                           colorProfile->get_colour_primaries());
+  if (input->has_nclx_color_profile()) {
+    auto colorProfile = input->get_color_profile_nclx();
+    coeffs = get_YCbCr_to_RGB_coefficients(colorProfile.get_matrix_coefficients(),
+                                           colorProfile.get_colour_primaries());
   }
 
   int r_cr = static_cast<int>(std::lround(256 * coeffs.r_cr));
@@ -361,14 +389,33 @@ Op_YCbCr420_to_RGB24::convert_colorspace(const std::shared_ptr<const HeifPixelIm
 
   uint32_t x, y;
   for (y = 0; y < height; y++) {
-    for (x = 0; x < width; x++) {
-      int yv = (in_y[y * in_y_stride + x]);
-      int cb = (in_cb[y / 2 * in_cb_stride + x / 2] - 128);
-      int cr = (in_cr[y / 2 * in_cr_stride + x / 2] - 128);
+    // Row pointers for input and output
+    const uint8_t* y_row = &in_y[y * in_y_stride];
+    const uint8_t* cb_row = &in_cb[(y / 2) * in_cb_stride];
+    const uint8_t* cr_row = &in_cr[(y / 2) * in_cr_stride];
+    uint8_t* out_row = &out_p[y * out_p_stride];
 
-      out_p[y * out_p_stride + 3 * x + 0] = clip_int_u8(yv + ((r_cr * cr + 128) >> 8));
-      out_p[y * out_p_stride + 3 * x + 1] = clip_int_u8(yv + ((g_cb * cb + g_cr * cr + 128) >> 8));
-      out_p[y * out_p_stride + 3 * x + 2] = clip_int_u8(yv + ((b_cb * cb + 128) >> 8));
+    int cb = 0;
+    int cr = 0;
+    int r_offset = 0;
+    int g_offset = 0;
+    int b_offset = 0;
+
+    for (x = 0; x < width; x++) {
+      // Update color offsets every other pixel
+      if ((x & 1) == 0) {
+        cb = cb_row[x / 2] - 128;
+        cr = cr_row[x / 2] - 128;
+        r_offset = ((r_cr * cr + 128) >> 8);
+        g_offset = ((g_cb * cb + g_cr * cr + 128) >> 8);
+        b_offset = ((b_cb * cb + 128) >> 8);
+      }
+
+      int yv = y_row[x];
+      uint8_t* rgb = &out_row[3 * x];
+      rgb[0] = clip_int_u8(yv + r_offset);
+      rgb[1] = clip_int_u8(yv + g_offset);
+      rgb[2] = clip_int_u8(yv + b_offset);
     }
   }
 
@@ -379,7 +426,8 @@ Op_YCbCr420_to_RGB24::convert_colorspace(const std::shared_ptr<const HeifPixelIm
 std::vector<ColorStateWithCost>
 Op_YCbCr420_to_RGB32::state_after_conversion(const ColorState& input_state,
                                              const ColorState& target_state,
-                                             const heif_color_conversion_options& options) const
+                                             const heif_color_conversion_options& options,
+                                             const heif_color_conversion_options_ext& options_ext) const
 {
   // this Op only implements the nearest-neighbor algorithm
 
@@ -398,11 +446,11 @@ Op_YCbCr420_to_RGB32::state_after_conversion(const ColorState& input_state,
     return {};
   }
 
-  int matrix = input_state.nclx_profile.get_matrix_coefficients();
+  int matrix = input_state.nclx.get_matrix_coefficients();
   if (matrix == 0 || matrix == 8 || matrix == 11 || matrix == 14) {
     return {};
   }
-  if (!input_state.nclx_profile.get_full_range_flag()) {
+  if (!input_state.nclx.get_full_range_flag()) {
     return {};
   }
 
@@ -428,6 +476,7 @@ Op_YCbCr420_to_RGB32::convert_colorspace(const std::shared_ptr<const HeifPixelIm
                                          const ColorState& input_state,
                                          const ColorState& target_state,
                                          const heif_color_conversion_options& options,
+                                         const heif_color_conversion_options_ext& options_ext,
                                          const heif_security_limits* limits) const
 {
   if (input->get_bits_per_pixel(heif_channel_Y) != 8 ||
@@ -450,11 +499,11 @@ Op_YCbCr420_to_RGB32::convert_colorspace(const std::shared_ptr<const HeifPixelIm
 
   // --- get conversion coefficients
 
-  auto colorProfile = input->get_color_profile_nclx();
   YCbCr_to_RGB_coefficients coeffs = YCbCr_to_RGB_coefficients::defaults();
-  if (colorProfile) {
-    coeffs = get_YCbCr_to_RGB_coefficients(colorProfile->get_matrix_coefficients(),
-                                           colorProfile->get_colour_primaries());
+  if (input->has_nclx_color_profile()) {
+    nclx_profile colorProfile = input->get_color_profile_nclx();
+    coeffs = get_YCbCr_to_RGB_coefficients(colorProfile.get_matrix_coefficients(),
+                                           colorProfile.get_colour_primaries());
   }
 
   int r_cr = static_cast<int>(std::lround(256 * coeffs.r_cr));
@@ -509,7 +558,8 @@ Op_YCbCr420_to_RGB32::convert_colorspace(const std::shared_ptr<const HeifPixelIm
 std::vector<ColorStateWithCost>
 Op_YCbCr420_to_RRGGBBaa::state_after_conversion(const ColorState& input_state,
                                                 const ColorState& target_state,
-                                                const heif_color_conversion_options& options) const
+                                                const heif_color_conversion_options& options,
+                                                const heif_color_conversion_options_ext& options_ext) const
 {
   // this Op only implements the nearest-neighbor algorithm
 
@@ -526,7 +576,7 @@ Op_YCbCr420_to_RRGGBBaa::state_after_conversion(const ColorState& input_state,
     return {};
   }
 
-  int matrix = input_state.nclx_profile.get_matrix_coefficients();
+  int matrix = input_state.nclx.get_matrix_coefficients();
   if (matrix == 0 || matrix == 8 || matrix == 11 || matrix == 14) {
     return {};
   }
@@ -563,6 +613,7 @@ Op_YCbCr420_to_RRGGBBaa::convert_colorspace(const std::shared_ptr<const HeifPixe
                                             const ColorState& input_state,
                                             const ColorState& target_state,
                                             const heif_color_conversion_options& options,
+                                            const heif_color_conversion_options_ext& options_ext,
                                             const heif_security_limits* limits) const
 {
   uint32_t width = input->get_width();
@@ -584,6 +635,23 @@ Op_YCbCr420_to_RRGGBBaa::convert_colorspace(const std::shared_ptr<const HeifPixe
   }
 
   if (has_alpha) {
+    if (input->get_width(heif_channel_Alpha) != width ||
+        input->get_height(heif_channel_Alpha) != height) {
+      return Error{
+        heif_error_Unsupported_feature,
+        heif_suberror_Unspecified,
+        "Color conversion cannot handle alpha images with sizes differing from the main image size."
+      };
+    }
+
+    if (input->get_bits_per_pixel(heif_channel_Alpha) != bpp) {
+      return Error{
+        heif_error_Unsupported_feature,
+        heif_suberror_Unspecified,
+        "Color conversion cannot handle alpha images with bits-per-pixel differing from the main image."
+      };
+    }
+
     if (auto err = outimg->add_plane(heif_channel_Alpha, width, height, bpp, limits)) {
       return err;
     }
@@ -609,11 +677,11 @@ Op_YCbCr420_to_RRGGBBaa::convert_colorspace(const std::shared_ptr<const HeifPixe
   bool full_range_flag = true;
   YCbCr_to_RGB_coefficients coeffs = YCbCr_to_RGB_coefficients::defaults();
 
-  auto colorProfile = input->get_color_profile_nclx();
-  if (colorProfile) {
-    full_range_flag = colorProfile->get_full_range_flag();
-    coeffs = get_YCbCr_to_RGB_coefficients(colorProfile->get_matrix_coefficients(),
-                                           colorProfile->get_colour_primaries());
+  if (input->has_nclx_color_profile()) {
+    nclx_profile colorProfile = input->get_color_profile_nclx();
+    full_range_flag = colorProfile.get_full_range_flag();
+    coeffs = get_YCbCr_to_RGB_coefficients(colorProfile.get_matrix_coefficients(),
+                                           colorProfile.get_colour_primaries());
   }
 
   float limited_range_offset = static_cast<float>(16 << (bpp - 8));
