@@ -35,12 +35,26 @@
 #endif
 
 
-constexpr inline uint32_t fourcc(const char* id)
+constexpr uint32_t four_bytes_to_uint32(uint8_t msb, uint8_t b, uint8_t c, uint8_t lsb)
 {
-  return (((((uint32_t) id[0])&0xFF) << 24) |
-          ((((uint32_t) id[1])&0xFF) << 16) |
-          ((((uint32_t) id[2])&0xFF) << 8) |
-          ((((uint32_t) id[3])&0xFF) << 0));
+  return (static_cast<uint32_t>(msb << 24) |
+          static_cast<uint32_t>(b << 16) |
+          static_cast<uint32_t>(c << 8) |
+          static_cast<uint32_t>(lsb));
+}
+
+constexpr uint16_t two_bytes_to_uint16(uint8_t msb, uint8_t lsb)
+{
+  return (static_cast<uint16_t>(msb << 8) |
+          static_cast<uint16_t>(lsb));
+}
+
+constexpr uint32_t fourcc(const char* id)
+{
+  return four_bytes_to_uint32(static_cast<uint8_t>(id[0]),
+                              static_cast<uint8_t>(id[1]),
+                              static_cast<uint8_t>(id[2]),
+                              static_cast<uint8_t>(id[3]));
 }
 
 std::string fourcc_to_string(uint32_t code);
@@ -83,19 +97,25 @@ inline uint8_t clip_int_u8(int x)
   return static_cast<uint8_t>(x);
 }
 
+inline uint16_t clip_int_u16(int32_t x, uint16_t maxi)
+{
+  if (x < 0) return 0;
+  if (x > maxi) return maxi;
+  return static_cast<uint16_t>(x);
+}
+
 
 inline uint16_t clip_f_u16(float fx, int32_t maxi)
 {
-  long x = (long int) (fx + 0.5f);
+  int32_t x = (int32_t) (fx + 0.5f);
   if (x < 0) return 0;
   if (x > maxi) return (uint16_t) maxi;
   return static_cast<uint16_t>(x);
 }
 
-
 inline uint8_t clip_f_u8(float fx)
 {
-  long x = (long int) (fx + 0.5f);
+  int32_t x = (int32_t) (fx + 0.5f);
   if (x < 0) return 0;
   if (x > 255) return 255;
   return static_cast<uint8_t>(x);
