@@ -47,7 +47,7 @@ ImageItem_AVIF::ImageItem_AVIF(HeifContext* ctx) : ImageItem(ctx)
 }
 
 
-Error ImageItem_AVIF::on_load_file()
+Error ImageItem_AVIF::initialize_decoder()
 {
   auto av1C_box = get_property<Box_av1C>();
   if (!av1C_box) {
@@ -57,12 +57,15 @@ Error ImageItem_AVIF::on_load_file()
 
   m_decoder = std::make_shared<Decoder_AVIF>(av1C_box);
 
+  return Error::Ok;
+}
+
+void ImageItem_AVIF::set_decoder_input_data()
+{
   DataExtent extent;
   extent.set_from_image_item(get_context()->get_heif_file(), get_id());
 
   m_decoder->set_data_extent(std::move(extent));
-
-  return Error::Ok;
 }
 
 
@@ -72,13 +75,13 @@ Result<std::vector<uint8_t>> ImageItem_AVIF::read_bitstream_configuration_data()
 }
 
 
-Result<std::shared_ptr<class Decoder>> ImageItem_AVIF::get_decoder() const
+Result<std::shared_ptr<Decoder>> ImageItem_AVIF::get_decoder() const
 {
   return {m_decoder};
 }
 
 
-std::shared_ptr<class Encoder> ImageItem_AVIF::get_encoder() const
+std::shared_ptr<Encoder> ImageItem_AVIF::get_encoder() const
 {
   return m_encoder;
 }
