@@ -25,6 +25,7 @@
 #include <vector>
 #include <string>
 #include <memory>
+#include <set>
 
 
 class ImageOverlay
@@ -97,7 +98,7 @@ public:
 
   // heif_compression_format get_compression_format() const override { return heif_compression_HEVC; }
 
-  Error on_load_file() override;
+  Error initialize_decoder() override;
 
   int get_luma_bits_per_pixel() const override;
 
@@ -108,16 +109,17 @@ public:
   heif_brand2 get_compatible_brand() const override;
 
   Result<Encoder::CodedImageData> encode(const std::shared_ptr<HeifPixelImage>& image,
-                                         struct heif_encoder* encoder,
-                                         const struct heif_encoding_options& options,
-                                         enum heif_image_input_class input_class) override
+                                         heif_encoder* encoder,
+                                         const heif_encoding_options& options,
+                                         heif_image_input_class input_class) override
   {
     return Error{heif_error_Unsupported_feature,
                  heif_suberror_Unspecified, "Cannot encode image to 'iovl'"};
   }
 
-  Result<std::shared_ptr<HeifPixelImage>> decode_compressed_image(const struct heif_decoding_options& options,
-                                                                  bool decode_tile_only, uint32_t tile_x0, uint32_t tile_y0) const override;
+  Result<std::shared_ptr<HeifPixelImage>> decode_compressed_image(const heif_decoding_options& options,
+                                                                  bool decode_tile_only, uint32_t tile_x0, uint32_t tile_y0,
+                                                                  std::set<heif_item_id> processed_ids) const override;
 
 
   // --- iovl specific
@@ -130,7 +132,8 @@ private:
 
   Error read_overlay_spec();
 
-  Result<std::shared_ptr<HeifPixelImage>> decode_overlay_image(const heif_decoding_options& options) const;
+  Result<std::shared_ptr<HeifPixelImage>> decode_overlay_image(const heif_decoding_options& options,
+                                                               std::set<heif_item_id> processed_ids) const;
 };
 
 
