@@ -499,7 +499,7 @@ heif_compression_format ImageItem_Tiled::get_compression_format() const
 }
 
 
-Error ImageItem_Tiled::on_load_file()
+Error ImageItem_Tiled::initialize_decoder()
 {
   auto heif_file = get_context()->get_heif_file();
 
@@ -541,8 +541,8 @@ Error ImageItem_Tiled::on_load_file()
   // TODO: remove when spec is final and old test images have been converted
   if (tilC_box->get_version() == 1) {
     auto propertiesResult = get_properties();
-    if (propertiesResult.error) {
-      return propertiesResult.error;
+    if (!propertiesResult) {
+      return propertiesResult.error();
     }
 
     m_tile_item->set_properties(*propertiesResult);
@@ -661,30 +661,33 @@ ImageItem_Tiled::add_new_tiled_item(HeifContext* ctx, const heif_tiled_image_par
 
 Error ImageItem_Tiled::add_image_tile(uint32_t tile_x, uint32_t tile_y,
                                      const std::shared_ptr<HeifPixelImage>& image,
-                                     struct heif_encoder* encoder)
+                                     heif_encoder* encoder)
 {
   auto item = ImageItem::alloc_for_compression_format(get_context(), encoder->plugin->compression_format);
 
   heif_encoding_options* options = heif_encoding_options_alloc(); // TODO: should this be taken from heif_context_add_tiled_image() ?
 
   Result<std::shared_ptr<HeifPixelImage>> colorConversionResult;
-  colorConversionResult = item->get_encoder()->convert_colorspace_for_encoding(image, encoder, *options, get_context()->get_security_limits());
-  if (colorConversionResult.error) {
+  colorConversionResult = item->get_encoder()->convert_colorspace_for_encoding(image, encoder,
+                                                                               options->output_nclx_profile,
+                                                                               &options->color_conversion_options,
+                                                                               get_context()->get_security_limits());
+  if (!colorConversionResult) {
     heif_encoding_options_free(options);
-    return colorConversionResult.error;
+    return colorConversionResult.error();
   }
 
-  std::shared_ptr<HeifPixelImage> colorConvertedImage = colorConversionResult.value;
+  std::shared_ptr<HeifPixelImage> colorConvertedImage = *colorConversionResult;
 
   Result<Encoder::CodedImageData> encodeResult = item->encode_to_bitstream_and_boxes(colorConvertedImage, encoder, *options, heif_image_input_class_normal); // TODO (other than JPEG)
   heif_encoding_options_free(options);
 
-  if (encodeResult.error) {
-    return encodeResult.error;
+  if (!encodeResult) {
+    return encodeResult.error();
   }
 
   const int construction_method = 0; // 0=mdat 1=idat
-  get_file()->append_iloc_data(get_id(), encodeResult.value.bitstream, construction_method);
+  get_file()->append_iloc_data(get_id(), encodeResult->bitstream, construction_method);
 
   auto& header = m_tild_header;
 
@@ -696,19 +699,19 @@ Error ImageItem_Tiled::add_image_tile(uint32_t tile_x, uint32_t tile_y,
   }
 
   uint64_t offset = get_next_tild_position();
-  size_t dataSize = encodeResult.value.bitstream.size();
+  size_t dataSize = encodeResult->bitstream.size();
   if (dataSize > 0xFFFFFFFF) {
     return {heif_error_Encoding_error, heif_suberror_Unspecified, "Compressed tile size exceeds maximum tile size."};
   }
   header.set_tild_tile_range(tile_x, tile_y, offset, static_cast<uint32_t>(dataSize));
-  set_next_tild_position(offset + encodeResult.value.bitstream.size());
+  set_next_tild_position(offset + encodeResult->bitstream.size());
 
   auto tilC = get_property<Box_tilC>();
   assert(tilC);
 
   std::vector<std::shared_ptr<Box>>& tile_properties = tilC->get_tile_properties();
 
-  for (auto& propertyBox : encodeResult.value.properties) {
+  for (auto& propertyBox : encodeResult->properties) {
 
     // we do not have to save ispe boxes in the tile properties as this is automatically synthesized
 
@@ -755,8 +758,9 @@ void ImageItem_Tiled::process_before_write()
 
 
 Result<std::shared_ptr<HeifPixelImage>>
-ImageItem_Tiled::decode_compressed_image(const struct heif_decoding_options& options,
-                                        bool decode_tile_only, uint32_t tile_x0, uint32_t tile_y0) const
+ImageItem_Tiled::decode_compressed_image(const heif_decoding_options& options,
+                                         bool decode_tile_only, uint32_t tile_x0, uint32_t tile_y0,
+                                         std::set<heif_item_id> processed_ids) const
 {
   if (decode_tile_only) {
     return decode_grid_tile(options, tile_x0, tile_y0);
@@ -796,17 +800,17 @@ ImageItem_Tiled::get_compressed_data_for_tile(uint32_t tx, uint32_t ty) const
 {
   // --- get compressed data
 
-  Error err = m_tile_item->init_decoder_from_item(0);
+  Error err = m_tile_item->initialize_decoder();
   if (err) {
     return err;
   }
 
   Result<std::vector<uint8_t>> dataResult = m_tile_item->read_bitstream_configuration_data();
-  if (dataResult.error) {
-    return dataResult.error;
+  if (!dataResult) {
+    return dataResult.error();
   }
 
-  std::vector<uint8_t>& data = dataResult.value;
+  std::vector<uint8_t> data = std::move(*dataResult);
   err = append_compressed_tile_data(data, tx, ty);
   if (err) {
     return err;
@@ -825,8 +829,8 @@ Result<std::shared_ptr<HeifPixelImage>>
 ImageItem_Tiled::decode_grid_tile(const heif_decoding_options& options, uint32_t tx, uint32_t ty) const
 {
   Result<DataExtent> extentResult = get_compressed_data_for_tile(tx, ty);
-  if (extentResult.error) {
-    return extentResult.error;
+  if (!extentResult) {
+    return extentResult.error();
   }
 
   m_tile_decoder->set_data_extent(std::move(*extentResult));
@@ -878,8 +882,8 @@ Error ImageItem_Tiled::get_coded_image_colorspace(heif_colorspace* out_colorspac
   uint32_t tx=0, ty=0; // TODO: find a tile that is defined.
 
   Result<DataExtent> extentResult = get_compressed_data_for_tile(tx, ty);
-  if (extentResult.error) {
-    return extentResult.error;
+  if (!extentResult) {
+    return extentResult.error();
   }
 
   m_tile_decoder->set_data_extent(std::move(*extentResult));
