@@ -23,30 +23,30 @@
 #include "security_limits.h"
 #include "common_utils.h"
 #include "decoder_dav1d.h"
-#include <memory>
 #include <cstring>
 #include <cassert>
 #include <cstdio>
+#include <deque>
 #include <limits>
-#include <utility>
 #include <string>
 
 #include <dav1d/version.h>
 #include <dav1d/dav1d.h>
 
+
 struct dav1d_decoder
 {
-  Dav1dSettings settings;
-  Dav1dContext* context;
-  Dav1dData data;
+  Dav1dSettings settings{};
+  Dav1dContext* context{};
+  std::deque<Dav1dData> queued_data;
   bool strict_decoding = false;
   std::string error_message;
 };
 
-static const char kEmptyString[] = "";
-static const char kSuccess[] = "Success";
+static constexpr char kEmptyString[] = "";
+static constexpr char kSuccess[] = "Success";
 
-static const int DAV1D_PLUGIN_PRIORITY = 150;
+static constexpr int DAV1D_PLUGIN_PRIORITY = 150;
 
 #define MAX_PLUGIN_NAME_LENGTH 80
 
@@ -77,7 +77,7 @@ static void dav1d_deinit_plugin()
 }
 
 
-static int dav1d_does_support_format(enum heif_compression_format format)
+static int dav1d_does_support_format(heif_compression_format format)
 {
   if (format == heif_compression_AV1) {
     return DAV1D_PLUGIN_PRIORITY;
@@ -88,7 +88,12 @@ static int dav1d_does_support_format(enum heif_compression_format format)
 }
 
 
-struct heif_error dav1d_new_decoder(void** dec)
+static int dav1d_does_support_format2(const heif_decoder_plugin_compressed_format_description* format)
+{
+  return dav1d_does_support_format(format->format);
+}
+
+heif_error dav1d_new_decoder2(void** dec, const heif_decoder_plugin_options* options)
 {
   auto* decoder = new dav1d_decoder();
 
@@ -105,18 +110,27 @@ struct heif_error dav1d_new_decoder(void** dec)
 
   if (dav1d_open(&decoder->context, &decoder->settings) != 0) {
     delete decoder;
-    struct heif_error err = {heif_error_Decoder_plugin_error, heif_suberror_Unspecified, kSuccess};
-    return err;
+    return {heif_error_Decoder_plugin_error, heif_suberror_Unspecified, kSuccess};
   }
 
-  memset(&decoder->data, 0, sizeof(Dav1dData));
+  if (options->num_threads) {
+    decoder->settings.n_threads = options->num_threads;
+  }
 
   *dec = decoder;
 
-  struct heif_error err = {heif_error_Ok, heif_suberror_Unspecified, kSuccess};
-  return err;
+  return heif_error_ok;
 }
 
+heif_error dav1d_new_decoder(void** dec)
+{
+  struct heif_decoder_plugin_options options;
+  options.format = heif_compression_AV1;
+  options.strict_decoding = false;
+  options.num_threads = 0;
+
+  return dav1d_new_decoder2(dec, &options);
+}
 
 void dav1d_free_decoder(void* decoder_raw)
 {
@@ -126,9 +140,15 @@ void dav1d_free_decoder(void* decoder_raw)
     return;
   }
 
-  if (decoder->data.sz) {
-    dav1d_data_unref(&decoder->data);
+  // free queued data
+
+  for (auto& pkt : decoder->queued_data) {
+    dav1d_data_unref(&pkt);
   }
+  decoder->queued_data.clear();
+
+  // free decoder context
+
   if (decoder->context) {
     dav1d_close(&decoder->context);
   }
@@ -139,70 +159,127 @@ void dav1d_free_decoder(void* decoder_raw)
 
 void dav1d_set_strict_decoding(void* decoder_raw, int flag)
 {
-  struct dav1d_decoder* decoder = (dav1d_decoder*) decoder_raw;
+  dav1d_decoder* decoder = (dav1d_decoder*) decoder_raw;
 
   decoder->strict_decoding = flag;
 }
 
-struct heif_error dav1d_push_data(void* decoder_raw, const void* frame_data, size_t frame_size)
+
+static heif_error push_pending_data_into_decoder(dav1d_decoder* decoder)
+{
+  while (!decoder->queued_data.empty()) {
+
+    // send data
+
+    int res = dav1d_send_data(decoder->context, &decoder->queued_data.front());
+
+    // decoder does not accept more data at this moment
+
+    if (res == DAV1D_ERR(EAGAIN)) {
+      break;
+    }
+
+    // Decoder has accepted data. Remove packet and check for error.
+
+    decoder->queued_data.pop_front();
+
+    if ((res < 0) && (res != DAV1D_ERR(EAGAIN))) {
+      return {
+        heif_error_Decoder_plugin_error,
+        heif_suberror_Unspecified,
+        kEmptyString
+      };
+    }
+  }
+
+  return heif_error_ok;
+}
+
+
+heif_error dav1d_push_data2(void* decoder_raw, const void* frame_data, size_t frame_size, uintptr_t user_data)
 {
   auto* decoder = (struct dav1d_decoder*) decoder_raw;
 
-  assert(decoder->data.sz == 0);
+  // --- copy input data into Dav1dData packet
 
-  uint8_t* d = dav1d_data_create(&decoder->data, frame_size);
+  Dav1dData packet{};
+
+  uint8_t* d = dav1d_data_create(&packet, frame_size);
   if (d == nullptr) {
-    struct heif_error err = {heif_error_Memory_allocation_error, heif_suberror_Unspecified, kSuccess};
-    return err;
+    return {heif_error_Memory_allocation_error, heif_suberror_Unspecified, kSuccess};
   }
 
   memcpy(d, frame_data, frame_size);
 
-  struct heif_error err = {heif_error_Ok, heif_suberror_Unspecified, kSuccess};
-  return err;
+  packet.m.user_data.data = (uint8_t*)user_data;
+
+  // --- put data into queue
+
+  decoder->queued_data.push_back(packet);
+
+  // --- push pending data to decoder
+
+  return push_pending_data_into_decoder(decoder);
 }
 
 
-struct heif_error dav1d_decode_next_image(void* decoder_raw, struct heif_image** out_img,
-                                          const heif_security_limits* limits)
+heif_error dav1d_push_data(void* decoder_raw, const void* frame_data, size_t frame_size)
+{
+  return dav1d_push_data2(decoder_raw, frame_data, frame_size, 0);
+}
+
+
+heif_error dav1d_decode_next_image2(void* decoder_raw, heif_image** out_img,
+                                    uintptr_t* out_user_data,
+                                    const heif_security_limits* limits)
 {
   auto* decoder = (struct dav1d_decoder*) decoder_raw;
 
-  struct heif_error err;
+  heif_error err;
 
-  Dav1dPicture frame;
-  memset(&frame, 0, sizeof(Dav1dPicture));
-
-  bool flushed = false;
+  Dav1dPicture frame{};
 
   for (;;) {
 
-    int res = dav1d_send_data(decoder->context, &decoder->data);
-    if ((res < 0) && (res != DAV1D_ERR(EAGAIN))) {
-      err = {heif_error_Decoder_plugin_error,
-             heif_suberror_Unspecified,
-             kEmptyString};
+    // --- send more pending data to decoder
+
+    err = push_pending_data_into_decoder(decoder);
+    if (err.code) {
       return err;
     }
 
-    res = dav1d_get_picture(decoder->context, &frame);
-    if (!flushed && res == DAV1D_ERR(EAGAIN)) {
-      if (decoder->data.sz == 0) {
-        flushed = true;
-      }
-      continue;
-    }
-    else if (res < 0) {
-      err = {heif_error_Decoder_plugin_error,
-             heif_suberror_Unspecified,
-             kEmptyString};
-      return err;
-    }
-    else {
+    // --- try to get decoded image
+
+    int res = dav1d_get_picture(decoder->context, &frame);
+
+    // We got a picture from the decoder. Continue with processing it.
+    if (res == 0) {
       break;
     }
+
+    // decoder wants more data, but queue is empty
+    if (res == DAV1D_ERR(EAGAIN) && decoder->queued_data.empty()) {
+      *out_img = nullptr;
+      return heif_error_ok;
+    }
+
+    // continue feeding more data from queue
+    if (res == DAV1D_ERR(EAGAIN)) {
+      continue;
+    }
+
+    // decoder error
+    if (res < 0) {
+      return {
+        heif_error_Decoder_plugin_error,
+        heif_suberror_Unspecified,
+        kEmptyString
+      };
+    }
   }
 
+  // --- convert image to heif_image
+
   heif_chroma chroma;
   heif_colorspace colorspace;
   switch (frame.p.layout) {
@@ -223,15 +300,19 @@ struct heif_error dav1d_decode_next_image(void* decoder_raw, struct heif_image**
       colorspace = heif_colorspace_monochrome;
       break;
     default: {
-      err = {heif_error_Decoder_plugin_error,
-             heif_suberror_Unspecified,
-             kEmptyString};
-      return err;
+      return {
+        heif_error_Decoder_plugin_error,
+        heif_suberror_Unspecified,
+        kEmptyString
+      };
     }
   }
 
+  if (out_user_data) {
+    *out_user_data = (uintptr_t)frame.m.user_data.data;
+  }
 
-  struct heif_image* heif_img = nullptr;
+  heif_image* heif_img = nullptr;
   err = heif_image_create(frame.p.w, frame.p.h,
                           colorspace,
                           chroma,
@@ -252,9 +333,6 @@ struct heif_error dav1d_decode_next_image(void* decoder_raw, struct heif_image**
   heif_image_set_nclx_color_profile(heif_img, &nclx);
 
 
-
-  // --- transfer data from Dav1dPicture to HeifPixelImage
-
   heif_channel channel2plane[3] = {
       heif_channel_Y,
       heif_channel_Cb,
@@ -289,7 +367,7 @@ struct heif_error dav1d_decode_next_image(void* decoder_raw, struct heif_image**
     size_t dst_stride;
     uint8_t* dst_mem = heif_image_get_plane2(heif_img, channel2plane[c], &dst_stride);
 
-    int bytes_per_pixel = (bpp + 7) / 8;
+    const int bytes_per_pixel = (bpp + 7) / 8;
 
     for (uint32_t y = 0; y < h; y++) {
       memcpy(dst_mem + y * dst_stride, data + y * stride, w * bytes_per_pixel);
@@ -301,21 +379,43 @@ struct heif_error dav1d_decode_next_image(void* decoder_raw, struct heif_image**
   *out_img = heif_img;
 
 
-  err = {heif_error_Ok, heif_suberror_Unspecified, kSuccess};
-  return err;
+  return {heif_error_Ok, heif_suberror_Unspecified, kSuccess};
 }
 
 
-struct heif_error dav1d_decode_image(void* decoder_raw, struct heif_image** out_img)
+heif_error dav1d_decode_next_image(void* decoder_raw, heif_image** out_img,
+                                   const heif_security_limits* limits)
+{
+  return dav1d_decode_next_image2(decoder_raw, out_img, nullptr, limits);
+}
+
+
+heif_error dav1d_decode_image(void* decoder_raw, struct heif_image** out_img)
 {
   auto* limits = heif_get_global_security_limits();
   return dav1d_decode_next_image(decoder_raw, out_img, limits);
 }
 
 
-static const struct heif_decoder_plugin decoder_dav1d
+heif_error dav1d_flush_data(void* decoder_raw)
+{
+  auto* decoder = (struct dav1d_decoder*) decoder_raw;
+
+  constexpr Dav1dData packet{}; // empty packet
+
+  // --- put data into queue
+
+  decoder->queued_data.push_back(packet);
+
+  // --- push pending data to decoder
+
+  return push_pending_data_into_decoder(decoder);
+}
+
+
+static const heif_decoder_plugin decoder_dav1d
     {
-        4,
+        5,
         dav1d_plugin_name,
         dav1d_init_plugin,
         dav1d_deinit_plugin,
@@ -326,11 +426,17 @@ static const struct heif_decoder_plugin decoder_dav1d
         dav1d_decode_image,
         dav1d_set_strict_decoding,
         "dav1d",
-        dav1d_decode_next_image
+        dav1d_decode_next_image,
+        /* minimum_required_libheif_version */ LIBHEIF_MAKE_VERSION(1,21,0),
+        dav1d_does_support_format2,
+        dav1d_new_decoder2,
+        dav1d_push_data2,
+        dav1d_flush_data,
+        dav1d_decode_next_image2
     };
 
 
-const struct heif_decoder_plugin* get_decoder_plugin_dav1d()
+const heif_decoder_plugin* get_decoder_plugin_dav1d()
 {
   return &decoder_dav1d;
 }
