@@ -116,8 +116,18 @@ public:
   Box_tkhd()
   {
     set_short_type(fourcc("tkhd"));
+
+    // set default flags according to ISO 14496-12
+    set_flags(Track_enabled | Track_in_movie | Track_in_preview);
   }
 
+  enum Flags : uint32_t {
+    Track_enabled = 0x01,
+    Track_in_movie = 0x02,
+    Track_in_preview = 0x04,
+    Track_size_is_aspect_ratio = 0x08
+  };
+
   std::string dump(Indent&) const override;
 
   const char* debug_box_name() const override { return "Track Header"; }
@@ -341,6 +351,8 @@ public:
 
   uint64_t get_total_duration(bool include_last_frame_duration);
 
+  uint32_t get_number_of_samples() const;
+
 protected:
   Error parse(BitstreamRange& range, const heif_security_limits*) override;
 
@@ -350,6 +362,46 @@ private:
 };
 
 
+// Composition Time to Sample Box
+class Box_ctts : public FullBox {
+public:
+  Box_ctts()
+  {
+    set_short_type(fourcc("ctts"));
+  }
+
+  std::string dump(Indent&) const override;
+
+  const char* debug_box_name() const override { return "Composition Time to Sample"; }
+
+  Error write(StreamWriter& writer) const override;
+
+  struct OffsetToSample {
+    uint32_t sample_count;
+    int32_t sample_offset;   // either uint32_t or int32_t, we assume that all uint32_t values will also fit into int32_t
+  };
+
+  int32_t get_sample_offset(uint32_t sample_idx);
+
+  void append_sample_offset(int32_t offset);
+
+  bool is_constant_offset() const;
+
+  void derive_box_version() override;
+
+  int32_t compute_min_offset() const;
+
+  uint32_t get_number_of_samples() const;
+
+protected:
+  Error parse(BitstreamRange& range, const heif_security_limits*) override;
+
+private:
+  std::vector<OffsetToSample> m_entries;
+  MemoryHandle m_memory_handle;
+};
+
+
 // Sample to Chunk Box
 class Box_stsc : public FullBox {
 public:
@@ -412,6 +464,8 @@ public:
 
   const std::vector<uint32_t>& get_offsets() const { return m_offsets; }
 
+  size_t get_number_of_chunks() const { return m_offsets.size(); }
+
   void patch_file_pointers(StreamWriter&, size_t offset) override;
 
 protected:
@@ -443,6 +497,8 @@ public:
 
   uint32_t get_fixed_sample_size() const { return m_fixed_sample_size; }
 
+  uint32_t num_samples() const { return m_sample_count; }
+
   const std::vector<uint32_t>& get_sample_sizes() const { return m_sample_sizes; }
 
   void append_sample_size(uint32_t size);
@@ -474,12 +530,19 @@ public:
 
   void add_sync_sample(uint32_t sample_idx) { m_sync_samples.push_back(sample_idx); }
 
+  // when this is set, the Box will compute whether it can be skipped
+  void set_total_number_of_samples(uint32_t num_samples);
+
+  // bool skip_box() const override { return m_all_samples_are_sync_samples; }
+
 protected:
   Error parse(BitstreamRange& range, const heif_security_limits*) override;
 
 private:
   std::vector<uint32_t> m_sync_samples;
   MemoryHandle m_memory_handle;
+
+  bool m_all_samples_are_sync_samples = false;
 };
 
 
@@ -516,10 +579,35 @@ private:
 };
 
 
+class Box_auxi : public FullBox {
+public:
+  Box_auxi()
+  {
+    set_short_type(fourcc("auxi"));
+  }
+
+  std::string dump(Indent&) const override;
+
+  const char* debug_box_name() const override { return "Auxiliary Info Type"; }
+
+  Error write(StreamWriter& writer) const override;
+
+  void set_aux_track_type_urn(const std::string& t) { m_aux_track_type = t; }
+
+  std::string get_aux_track_type_urn() const { return m_aux_track_type; }
+
+protected:
+  Error parse(BitstreamRange& range, const heif_security_limits*) override;
+
+private:
+  std::string m_aux_track_type;
+};
+
+
 struct VisualSampleEntry {
   // from SampleEntry
   //const unsigned int(8)[6] reserved = 0;
-  uint16_t data_reference_index;
+  uint16_t data_reference_index = 1;
 
   // VisualSampleEntry
 
@@ -753,6 +841,8 @@ public:
 
   uint8_t get_sample_size(uint32_t idx);
 
+  bool have_samples_constant_size() const { return m_default_sample_info_size != 0; }
+
   uint32_t get_num_samples() const { return m_num_samples; }
 
   std::string dump(Indent&) const override;
@@ -788,12 +878,12 @@ public:
 
   uint32_t get_aux_info_type_parameter() const { return m_aux_info_type_parameter; }
 
-  void add_sample_offset(uint64_t offset);
+  void add_chunk_offset(uint64_t offset);
 
-  // This will be 1 if all infos are written contiguously
-  size_t get_num_samples() const { return m_sample_offset.size(); }
+  // If this is 1, the SAI data of all samples is written contiguously in the file.
+  size_t get_num_chunks() const { return m_chunk_offset.size(); }
 
-  uint64_t get_sample_offset(uint32_t idx) const;
+  uint64_t get_chunk_offset(uint32_t idx) const;
 
   std::string dump(Indent&) const override;
 
@@ -813,13 +903,42 @@ private:
   bool m_need_64bit = false;
   mutable uint64_t m_offset_start_pos;
 
-  // If sample_offset==1, all samples are stored contiguous in the file
-  std::vector<uint64_t> m_sample_offset;
+  // If |chunk_offset|==1, the SAI data of all samples is stored contiguously in the file
+  std::vector<uint64_t> m_chunk_offset;
 
   MemoryHandle m_memory_handle;
 };
 
 
+class Box_sdtp : public FullBox {
+public:
+  Box_sdtp()
+  {
+    set_short_type(fourcc("sdtp"));
+  }
+
+  std::string dump(Indent&) const override;
+
+  const char* debug_box_name() const override { return "Independent and Disposable Samples"; }
+
+  // Error write(StreamWriter& writer) const override;
+
+  uint8_t get_is_leading(uint32_t sampleIdx) const { return (m_sample_information[sampleIdx] >> 6) & 3; }
+
+  uint8_t get_depends_on(uint32_t sampleIdx) const { return (m_sample_information[sampleIdx] >> 4) & 3; }
+
+  uint8_t get_is_depended_on(uint32_t sampleIdx) const { return (m_sample_information[sampleIdx] >> 2) & 3; }
+
+  uint8_t get_has_redundancy(uint32_t sampleIdx) const { return (m_sample_information[sampleIdx]) & 3; }
+
+protected:
+  Error parse(BitstreamRange& range, const heif_security_limits*) override;
+
+private:
+  std::vector<uint8_t> m_sample_information;
+};
+
+
 class Box_tref : public Box
 {
 public:
@@ -859,4 +978,58 @@ private:
 };
 
 
+// Edit List container
+class Box_edts : public Box_container {
+public:
+  Box_edts() : Box_container("edts") {}
+
+  const char* debug_box_name() const override { return "Edit List Container"; }
+};
+
+
+class Box_elst : public FullBox
+{
+public:
+  Box_elst()
+  {
+    set_short_type(fourcc("elst"));
+  }
+
+  enum Flags {
+    Repeat_EditList = 0x01
+  };
+
+  std::string dump(Indent&) const override;
+
+  const char* debug_box_name() const override { return "Edit List"; }
+
+  void enable_repeat_mode(bool enable);
+
+  bool is_repeat_mode() const { return get_flags() & Flags::Repeat_EditList; }
+
+  struct Entry {
+    uint64_t segment_duration = 0;
+    int64_t media_time = 0;
+    int16_t media_rate_integer = 1;
+    int16_t media_rate_fraction = 0;
+  };
+
+  void add_entry(const Entry&);
+
+  size_t num_entries() const { return m_entries.size(); }
+
+  Entry get_entry(uint32_t entry) const { return m_entries[entry]; }
+
+protected:
+  Error parse(BitstreamRange& range, const heif_security_limits*) override;
+
+  Error write(StreamWriter& writer) const override;
+
+public:
+  void derive_box_version() override;
+
+private:
+  std::vector<Entry> m_entries;
+};
+
 #endif //SEQ_BOXES_H
