@@ -131,7 +131,7 @@ if [ ! -z "$TESTS" ]; then
     CMAKE_OPTIONS="--preset=testing"
 fi
 if [ -z "$FUZZER" ] && [ -z "$TESTS" ]; then
-    CMAKE_OPTIONS="--preset=release -DENABLE_PLUGIN_LOADING=OFF"
+    CMAKE_OPTIONS="--preset=release -DENABLE_PLUGIN_LOADING=OFF -DWITH_OPENJPH_ENCODER=OFF"
 fi
 	
 if [ "$CURRENT_OS" = "osx" ] ; then
@@ -153,6 +153,9 @@ CMAKE_OPTIONS="$CMAKE_OPTIONS -DCMAKE_INSTALL_PREFIX=${BUILD_ROOT}/dist"
 # turn on warnings-as-errors
 CMAKE_OPTIONS="$CMAKE_OPTIONS -DCMAKE_COMPILE_WARNING_AS_ERROR=1"
 
+# compilation mode
+CMAKE_OPTIONS="$CMAKE_OPTIONS -DCMAKE_BUILD_TYPE=Release"
+
 
 if [ ! -z "$FUZZER" ] && [ "$CURRENT_OS" = "linux" ]; then
     export ASAN_SYMBOLIZER="$BUILD_ROOT/clang/bin/llvm-symbolizer"
@@ -174,6 +177,7 @@ if [ -z "$EMSCRIPTEN_VERSION" ] && [ -z "$CHECK_LICENSES" ] && [ -z "$TARBALL" ]
         ${BIN_WRAPPER} ./examples/heif-dec${BIN_SUFFIX} --list-decoders
 
         echo "Dumping information of sample file ..."
+        #${BIN_WRAPPER} gdb -batch -ex "run" -ex "bt" --args ./examples/heif-info${BIN_SUFFIX} --dump-boxes examples/example.heic
         ${BIN_WRAPPER} ./examples/heif-info${BIN_SUFFIX} --dump-boxes examples/example.heic
         if [ ! -z "$WITH_GRAPHICS" ] && [ ! -z "$WITH_HEIF_DECODER" ]; then
             echo "Converting sample HEIF file to JPEG ..."
