@@ -34,7 +34,7 @@
 
 #include "uncompressed_decode.h"
 
-void check_image_size_rgb16(struct heif_context *&context, int expect_alpha) {
+void check_image_size_rgb16(heif_context *&context, int expect_alpha) {
   heif_image_handle *handle = get_primary_image_handle(context);
   heif_image *img = get_primary_image(handle);
 
@@ -96,7 +96,7 @@ TEST_CASE("check image size 16 bit RGB") {
   heif_context_free(context);
 }
 
-void check_image_content_rgb16(struct heif_context *&context) {
+void check_image_content_rgb16(heif_context *&context) {
   heif_image_handle *handle = get_primary_image_handle(context);
   heif_image *img = get_primary_image(handle);
 
